#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   lib.sh of /CoreOS/samba/Library/samba-bits
#   Description: Library for handling admin tasks
#   Author: Robin Hack <rhack@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2014 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   library-prefix = samba-bits
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head1 NAME

samba/samba-bits - Library for handling common administration tasks.

=head1 DESCRIPTION

This  library contains set of functions which helps to easy write tests for samba component.

=cut

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Variables
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head1 VARIABLES

=over

=item SB_DISABLE_NEW_CONST

If this variable is defined, constuctor and destructor are not called.

=item WORKGROUP="EXAMPLETRUST"

Workgroup name.

For more information about this option please take a look at man smb.conf, section workgroup (G).

=item REALM="EXAMPLETRUST.EXAMPLE"

Full name of Active Directory tree.

=item PDC_NBNAME="x64-win-2008-PDC"

NetBIOS name of server Primary Domain Controller.

=item PDC_IP="192.168.1.1"

IPv4 address of server Primary Domain Controller.

=item PDC_LOGIN="administrator"

Login to server Primary Domain Controller.

=item PDC_PASSWORD="_VERY_SECRET_STING_"

Password to server Primary Domain Controller.

=item SECURITY="ADS"

Possible values: ADS, PDC

For more information about this option please take a look at man smb.conf, section security (G).

=item PDC_CHILD_IP

IPv4 address of server child Primary Domain Controller.

=item PDC_CHILD_NBNAME

NetBIOS name of the child server.

=item BDC_IP

Address of Backup Domain Controller.

=item BDC_NBNAME 

NetBIOS name of Backup Domain Controller.

=item SB_SSH_KEY_PATH

Path to private ssh key.

=back

=cut

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sambaLibraryLoaded
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head1 FUNCTIONS

=head2 sambaLibraryLoaded

Check if the library is loaded correctly.

=over

=back

Always returns 0 for now.

=cut

sambaLibraryLoaded() {
    return 0
}

# Author: Ales Zelinka <azelinka@redhat.com>

#this a set of functions used for samba testing. It helps keep samba test 
#small and maintenable. It relies on some of these variables to be defined:
#REALM
#WORKGROUP
#PDC_IP
#PDC_NBNAME
#PDC_LOGIN
#PDC_PASSWORD
#BDC_IP
#BDC_NBNAME
#TRUST_...
#..
#and rhts/rhtslib environment set

#usage: source this file from rhtslib-enabled test

# Copyright (c) 2008 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

SB_VERSION="1.4"


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_save_and_set_clean_env()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_save_and_set_clean_env

Function is primarily called by constructor. This function tries to establish clean environment for samba test. Current configuration files are backed up and configurations are set to default values. Samba cache files are cleaned. All users and their passwords are removed. Set address of localhost in /etc/samba/lmhosts. Services smb, nmb, winbind and ntpd are restarted.

=cut

sb_save_and_set_clean_env(){
    rlServiceStop smb nmb winbind chronyd
    rlIsRHEL '<8' && rlServiceStop ntpd
    # used with authselect (introduced in RHEL8) with mkhomedir parameter for pam_winbind
    rlIsRHEL '>=8' && rlServiceStop oddjobd
    rlFileBackup --namespace samba_bits "/etc/resolv.conf" "/etc/samba/lmhosts" "/etc/hosts"
    rlFileBackup --namespace samba_bits "/etc/samba/smbpasswd"
    rlFileBackup --namespace samba_bits "/etc/passwd" "/etc/shadow" "/etc/group" "/etc/gshadow"
    rlFileBackup --namespace samba_bits "/etc/pam.d/"
    rlFileBackup --namespace samba_bits "/etc/nsswitch.conf" `readlink -f /etc/nsswitch.conf` "/etc/krb5.conf"
    rlFileBackup --namespace samba_bits "/etc/sysconfig/authconfig"
    rlFileBackup --namespace samba_bits "/etc/login.defs"
    rlFileBackup --namespace samba_bits "/etc/sysconfig/network"

    sb_config_smb_default
    sb_clean_cache
    sb_clean_etc_samba

    cat > /etc/samba/lmhosts <<-EOF
127.0.0.1 localhost
EOF
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_restore_and_set_clean_env()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_restore_and_set_clean_env

Function is primarily called by destructor. Services smb, nmb, winbind, ntpd are stopped. Samba cash files are cleaned. All users and their passwords are removed. Restore backed up configuration files to their original location.

=cut

# This function is called by destructor.
sb_restore_and_set_clean_env(){
    rlServiceStop smb nmb winbind chronyd
    rlIsRHEL '<8' && rlServiceStop ntpd
    sb_clean_cache
    sb_clean_etc_samba

    for file in "/var/lib/samba/browse.dat" "/var/lib/samba/smb_krb5" "/var/lib/samba/smb_krb5/krb5.conf.ZELGROUP"; do
        if [ -f $file ]; then
            rm -f $file;
        fi
    done

    if ! rlIsRHEL "5"; then
        rm -rf "/var/lib/samba/printing"
        rm -rf "/var/lib/samba/winbindd_privileged/pipe"
    fi

    if rlIsRHEL "6"; then
        rm -rf "/var/nmbd";
    elif rlIsRHEL ">=7"; then
        rm -f  "/var/lib/samba/private/secrets.ldb";
        rm -rf "/var/lib/samba/lock/msg.lock" "/var/lib/samba/private/msg.sock";
    fi

    hwclock -s && rlLog "Set the time back to normal (after syncing with AD)"

    # For cases when machine is configured as PDC
    setsebool samba_domain_controller=0
    setsebool samba_enable_home_dirs=0

    if rlIsRHEL '>=8' && rpm -q authselect; then
        if ! authselect current | grep sssd; then
            authselect select sssd --force
            rlLog "Default authselect profile (sssd)"
        fi
    fi

    rlFileRestore --namespace samba_bits
    rlServiceRestore smb nmb winbind chronyd
    rlIsRHEL '<8' && rlServiceRestore ntpd
    rlIsRHEL '>=8' && rlServiceRestore oddjobd
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_get_separator()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_get_separator

Find and return the value of the winbind separator. Default value is " \ ".
    
=over

=back

Returns value of separator.

=cut

sb_get_separator(){
   local SEP=`testparm  -s  2>/dev/null | grep "winbind separator" |cut -d = -f 2 |sed 's/ //g'`
   [ -z "$SEP" ] && SEP="\\"
   echo $SEP
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_get_branch()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_get_branch

=over

=back

Name of installed component are returned. If the component is dual in RHEL, it should return one of: samba, samba3x or samba4.    

=cut

sb_get_branch(){
    rpm -q samba3x-common > /dev/null && echo "samba3x" || echo "samba"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_get_version()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_get_version

=over

=back

Version of samba package are returned.

=cut

sb_get_version(){
    echo "$(sb_get_major_version).$(sb_get_minor_version)"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_get_minor_version()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_get_minor_version

=over

=back

Function returns minor version of Samba package. For example: Have a package samba-4.1.9-4.fc20.x86_64, minor version is 1.

=cut

sb_get_minor_version(){
    local PACKAGE=`sb_get_branch`
    local VERSION=`rpm -q --queryformat="%{version}" $PACKAGE`
    local MINOR_VERSION=`echo $VERSION |cut -d . -f 2`
    echo $MINOR_VERSION
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_get_major_version()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_get_major_version

=over

=back

Function returns major version of Samba package. For example: Have a package samba-4.1.9-4.fc20.x86_64, major version is 4.

=cut

sb_get_major_version(){
    local PACKAGE=`sb_get_branch`
    local VERSION=`rpm -q --queryformat="%{version}" $PACKAGE`
    local MAJOR_VERSION=`echo $VERSION |cut -d . -f 1`
    echo $MAJOR_VERSION
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_get_nbname()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_get_nbname

=over

=back

Return NetBIOS name of machine where test runs.

=cut

sb_get_nbname(){
    local NBNAME=`hostname -s`
    NBNAME=${NBNAME:0:15}
    echo $NBNAME
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_get_config()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_get_config [NAME FILENAME|]

Try to include configuration file for test.

=over 

=item NAME

Name of the test. If parameter is not set, it takes test name from global variable $TEST from BeakerLib.

=item FILENAME

Name of the configuration file. If parameter is not set, it is set to "win-config-${NAME}.sh"
=back

Returns 0 if configuration file was included. Returns 1 if test specific config was not found or is empty.

=cut

sb_get_config(){
    local NAME=${1:-$(basename "${TEST}")}
    local FILENAME=${2:-win-config-${NAME}.sh}

    if [ -s "./${FILENAME}" ]; then
        rlRun ". ./${FILENAME}" 0 "including config from LOCALFILE ${FILENAME}"
        return 0
    # Not so clever solution...
    elif [ -s "./win-config.sh" ]; then
        rlRun ". ./win-config.sh" 0 "including config from LOCALFILE win-config.sh"
        return 0
    else
        rlLogWarning "test specific config not found or empty"
        return 1
    fi
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_config_smb_default()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_config_smb_default

Set /etc/samba/smb.conf to known state.

=over

=back

=cut

#restores very basic smb.conf
#removed s "max log size = 50" and "cups options = raw"
sb_config_smb_default(){
    rlFileBackup --namespace samba_bits  "/etc/samba/smb.conf"
    cat > /etc/samba/smb.conf <<-EOF
[global]
	workgroup = MYGROUP
	server string = Samba Server Version %v
	# because badlock we need to have it for all samba versions
	server signing = auto

[homes]
	comment = Home Directories
	read only = No
	browseable = No

[printers]
	comment = All Printers
	path = /var/spool/samba
	printable = Yes
	browseable = No
EOF
    if rlIsRHEL ">=7"; then
        sb_config_add "global" "winbind request timeout = 120"
    fi
    rlLog "smb.conf modified"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_config_smb_conf_member()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_config_smb_conf_member

Configures Samba to be able join an Active Directory domain as a member.

=over

=back

=cut

sb_config_smb_conf_member(){
    # backup only when constructor is not called (otherwise smb.conf file is already backed up)
    [ ! -z $SB_DISABLE_NEW_CONST ] && rlFileBackup --namespace samba_bits "/etc/samba/smb.conf"
    local NBNAME=`sb_get_nbname`
    cat > /etc/samba/smb.conf <<-EOF
[global]
realm = $REALM
server signing = auto
netbios name = $NBNAME
workgroup = $WORKGROUP
security = $SECURITY
create krb5 conf = no
password server = *
wins server = $PDC_IP, $BDC_IP
encrypt passwords = yes
EOF
    if rlIsRHEL ">=7"; then
        sb_config_add "global" "winbind request timeout = 120"
    fi

    #idmapping format changed in 3.6
    if [ "`sb_get_major_version`" -lt 4 -a "`sb_get_minor_version`" -lt 6 ] ; then
        sb_config_add "global" "idmap uid = 10000-20000"
        sb_config_add "global" "idmap gid = 10000-20000"
    else
        sb_config_add "global" 'idmap config * : range = 10000-20000'  
    fi
    testparm -s
    rlLog "smb.conf modified"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_config_smb_conf_pdc()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_config_smb_conf_pdc

Function configures Samba as primary domain controller and also takes care about SELinux booleans.

=over

=back

=cut

sb_config_smb_conf_pdc(){
    # backup only when constructor is not called (otherwise it backups smb.conf file)
    [ ! -z $SB_DISABLE_NEW_CONST ] && rlFileBackup --namespace samba_bits "/etc/samba/smb.conf"

    [ -n "$1" ] && local NBNAME=$1 || local NBNAME="zelserver"
    [ -z "$WORKGROUP" ] && local WORKGROUP="ZELGROUP" 
    cat > /etc/samba/smb.conf <<-EOF
[global]
workgroup = $WORKGROUP
netbios name = $NBNAME
security = user
server signing = auto
os level = 65
preferred master = yes
domain master = yes
local master = yes
domain logons = yes
wins support = yes

logon home = \\%N\%U
logon drive = H:

passwd program = /usr/bin/passwd %u
passwd chat = *New*password* %n\n *Retype*new*password* %n\n *passwd:*all*authentication*tokens*updated*successfully*
unix password sync = Yes
add user script = /usr/sbin/useradd -m %u
delete user script = /usr/sbin/userdel -r %u
add group script = /usr/sbin/groupadd %g
delete group script = /usr/sbin/groupdel %g
add user to group script = /usr/sbin/groupmod -A %u %g
delete user from group script = /usr/sbin/groupmod -R %u %g
add machine script = /usr/sbin/useradd -s /bin/false -d /var/lib/nobody %u

[netlogon]
comment = Network Logon Service
path = /tmp/sb-nl-$NBNAME
guest ok = Yes

[homes]
comment = Home Directories
admin users = root
read only = No
browseable = No

EOF
    if rlIsRHEL ">=7"; then
        sb_config_add "global" "winbind request timeout = 120"
    fi
    mkdir -p "/tmp/sb-nl-$NBNAME"
    chmod a+rwx "/tmp/sb-nl-$NBNAME"
    setsebool samba_domain_controller=1
    setsebool samba_enable_home_dirs=1
    rlLog "smb.conf modified"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_config_lmhosts()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_config_lmhosts

Function sets /etc/samba/lmhosts to known state according to global parameters REALM, PDC_IP+PDC_NBNAME, BDC_IP+BDC_NBNAME and PDC_CHILD_IP and sets address of localhost.

=over

=back

=cut

sb_config_lmhosts(){
    cat > /etc/samba/lmhosts <<-EOF
127.0.0.1 localhost

$PDC_IP $(echo $PDC_NBNAME | awk '{ print toupper($0) }')
EOF

    if [ -n "$BDC_IP" ] ; then
        cat >> /etc/samba/lmhosts <<-EOF
$BDC_IP $(echo $BDC_NBNAME | awk '{ print toupper($0) }')
EOF
    fi

#TODO: Not sure if this is correct configuration for Child domain
    if [ -n "$PDC_CHILD_IP" ] ; then
        cat >> /etc/samba/lmhosts <<-EOF
$PDC_CHILD_IP CHILD.$REALM#1c
$PDC_CHILD_IP CHILD.$REALM#1b
$PDC_CHILD_IP CHILD#1b
$PDC_CHILD_IP CHILD#1c
EOF
    fi
    rlLog "lmhosts modified"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_config_hosts()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_config_hosts

Configuration file /etc/hosts is set to known state according to global parameters PDC_NBNAME, BDC_NBNAME, PDC_CHILD_NBNAME and machine IP address and NBNAME.

=over

=back

=cut

sb_config_hosts(){
    local IP=$(ip a s dev $(ip r | awk '/^default/{printf $5; }') | awk '{ if ($1 == "inet") { if (substr($2,0,3) == "10.") { printf substr($2, 0, index($2, "/") - 1); }} }')
    local NBNAME=`sb_get_nbname`

    # commenting out record with machine hostname, we will add different record to be recognized by AD DNS
    sed -e "/[\d\.]*\s*$(hostname)\s$(hostname -s)/ s/^/#/" -i /etc/hosts

    cat >> /etc/hosts <<-EOF
$IP    $NBNAME.$REALM $NBNAME 
$PDC_IP   $PDC_NBNAME.$REALM $PDC_NBNAME
EOF
    if [ -n "$BDC_NBNAME" ] ; then
        echo "$BDC_IP    $BDC_NBNAME.$REALM" >> /etc/hosts
    fi  
    if [ -n "$PDC_CHILD_NBNAME" ] ; then
        echo "$PDC_CHILD_IP    $PDC_CHILD_NBNAME.CHILD.$REALM" >> /etc/hosts
    fi  
    rlLog "hosts modified"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_config_krb5_conf()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_config_krb5_conf

Configuration file /etc/krb5.conf is set to know state according to global parameters PDC_CHILD_ID, REALM, PDC_IP.

Also adds some compatibility settings for older windows versions.

=over

=back

=cut

sb_config_krb5_conf(){
    rlFileBackup --namespace samba_bits "/etc/krb5.conf"
    #rlLogInfo "workaround for the old RHEL5.2.z SElinux policy - creating /var/cache/samba/smb_krb5"
    #mkdir -p /var/cache/samba/smb_krb5
    #restorecon -R -v /var/cache/samba
    cat > /etc/krb5.conf <<-EOF
[libdefaults]
default_realm = $REALM
#this is needed for w2k8 join and should work with older wins too
default_tgs_enctypes = RC4-HMAC DES-CBC-CRC DES-CBC-MD5
default_tkt_enctypes = RC4-HMAC DES-CBC-CRC DES-CBC-MD5
preferred_enctypes = RC4-HMAC DES-CBC-CRC DES-CBC-MD5
kdc_timeout = 120

[realms]
$REALM = {
    kdc = $PDC_IP
}

EOF
    if [ -n "$PDC_CHILD_IP" ] ; then
        cat >> /etc/krb5.conf <<-EOF
CHILD.$REALM = {
   kdc = $PDC_CHILD_IP
}
EOF
    fi
    rlLog "krb5.conf modified"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_config_nsswitch_conf()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_config_nsswitch_conf

Configuration file /etc/nsswitch.conf is set to know state. 

Adds winbind support.

=over

=back

=cut

sb_config_nsswitch_conf(){
    rlFileBackup --namespace samba_bits "/etc/nsswitch.conf" `readlink -f /etc/nsswitch.conf`
    if rlIsRHEL '<8'; then
        cat > /etc/nsswitch.conf <<-EOF
passwd:     files winbind
shadow:     files
group:      files winbind

hosts:      files dns wins

bootparams: nisplus [NOTFOUND=return] files
ethers:     files
netmasks:   files
networks:   files
protocols:  files
rpc:        files
services:   files
netgroup:   nisplus
publickey:  nisplus
automount:  files nisplus
aliases:    files nisplus
EOF
        rlLog "nsswitch.conf modified"
    else
        if ! authselect current | grep winbind ; then
            rlRun "authselect select winbind --force" 0 "Choosing winbind profile for PAM and nsswitch config."
        else
            rlLog "Winbind profile via authselect is already selected, skipping ..."
        fi
    fi
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_useradd()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_useradd [USERNAME PWD|]

Function creates a new samba user.

=over

=item USERNAME

If parameter is not set, it is set to value smbuser.

=item PWD

If parameter is not set, it is set to value of global parameter $USERNAME.

=back

Return 0 if ends correctly.
	
=cut

sb_useradd(){
    local USERNAME=${1:-smbuser}
    local PWD=${2:-$USERNAME}
    if ! getent passwd "$USERNAME" ; then
        useradd $USERNAME
    fi
    echo -e "$PWD\n$PWD" | smbpasswd -s -a "$USERNAME"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_pdc_time_synchro()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_pdc_time_synchro 

This function is synchronizing time with PDC.

=over

=back
	
=cut

sb_pdc_time_synchro(){
    #redirect the output to workaround AVC denial (because of RHTS init_rc context, not a policy bug)
    rlLog "synchronizing time with ADS"
    (rpm -q ntpdate || rpm -q ntp) && {
        local OUTPUT=`ntpdate $PDC_IP 2>&1`
    } || local OUTPUT=`chronyd -q "server $PDC_IP iburst" 'maxdistance 16'` 2>&1
    echo $OUTPUT
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_config_resolv_conf()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_config_resolv_conf 

Function sets nameserver and domain according to global variables PDC_IP and REALM.

=over

=back
	
=cut

sb_config_resolv_conf(){
    # Remove whole line containg "search $DOMAIN" and replace it with AD settings
    sed -i "/search/c\domain $REALM\nnameserver $PDC_IP" /etc/resolv.conf
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_config_pam()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_config_pam 

PAM for the most common setup with winbind enabled.

=over

=back
	
=cut

#configure pam for the most common setup with winbind
#!!WARNING! using authconfig/authselect - this will rewrite other configs as well
sb_config_pam(){
    if rlIsRHEL '<8'; then
        rlRun "authconfig --enablewinbindauth --enablemkhomedir --winbindtemplateshell=/bin/bash --enablewinbindoffline  --update" 0 "authconfig-uring pam"
    else
        sb_config_add "global" "template shell = /bin/bash"
        sb_config_add "global" "winbind offline logon = yes"

        if ! authselect current | grep winbind; then
            rlRun "authselect select winbind --force" 0 "Choosing winbind profile for PAM and nsswitch config."
        fi

        if rpm -q oddjob-mkhomedir; then
            if ! authselect current | grep with-mkhomedir; then
                rlRun "authselect enable-feature with-mkhomedir" 0 "Enable making home directory for AD users"
            else
                rlLog "Feature \"with-mkhomedir\" is already selected, skipping ..."
            fi
            rlLog "Package oddjob-mkhomedir is installed"
            rlServiceStart oddjobd
        else
            rlLog "Package oddjob-mkhomedir is not installed, skipping ..."
        fi
    fi
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_torture()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_torture TEST [ID_RSA HOST SMB_USER HOST_USER SMB_USERPASS|]

Run upstream remote test from samba upstream testsuite.

=over

=item TEST

Name of test.

=item ID_RSA

Name of ssh key.

=item HOST

IP address of host server.

=item SMB_USER

Name of samba user.

=item HOST_USER

Name of user on remote host machine.

=item SMB_USERPASS

Samba user password.

=back
	
=cut

sb_torture(){
    local TEST="$1"
    local ID_RSA="${2:-"id_rsa"}"
    local HOST="${3:-"127.0.0.1"}"
    local SMB_USER="${4:-"smbuser"}"
    local HOST_USER="${5:-"root"}"
    local SMB_USERPASS="${6:-"$4"}"

    local MYIP=$(sb_get_ip)

    sb_run_remote_command \
            "/srv/samba/bin/smbtorture //${MYIP}/sambak $TEST -U ${SMB_USER}%${SMB_USERPASS}" \
            "${HOST}" \
            "${HOST_USER}" \
            "./" \
            "${ID_RSA}"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_run_remote_command()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_run_remote_command [CMD HOST USER KEYDIR KEY|]

This function runs command on the windows infrastructure. 

=over

=item CMD

Path to smbtorture on the server.

=item HOST

IPv4 address.

=item USER

User name.

=item KEYDIR

Path to key.

=item KEY

Name of key.

=back
	
Return open ssh connection to windows machine.
	
=cut

# this function run remote command on windows which run ssh server
sb_run_remote_command(){
    local CMD="$1"
    local HOST="${2:-$PDC_IP}"
    local USER="${3:-$PDC_LOGIN}"
    local KEYDIR="${4:-"./"}"
    local KEY=${5:-"id_rsa"}

    rlRun "ssh -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null -o ConnectTimeout=60 \
             -i ${KEYDIR}/${KEY} ${USER}@${HOST} \"${CMD}\""
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_config_join()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_config_join 

Function configures machine to be member of Active Directory.

=over

=back
	
=cut

#usage: sb_config_join
sb_config_join(){
    rlRun "ping -c 1 -w 2 -q $PDC_IP" 0 "PDC for $REALM (expected to be at $PDC_IP) responds to ping"
    [ "x$PDC_CHILD_IP" != "x" ] && rlRun "ping -c 1 -w 2 -q $PDC_CHILD_IP" 0 "PDC for CHILD.$REALM (expected to be at $PDC_CHILD_IP) responds to ping"
    [ "x$BDC_IP" != "x" ] && rlRun "ping -c 1 -w 2 -q $BDC_IP" 0 "BDC for $REALM (expected to be at $BDC_IP) responds to ping"
    if rlIsRHEL 5 && rpm -q samba > /dev/null; then
        rlServiceStop smb winbind ntpd chronyd
    else
        rlServiceStop winbind smb nmb chronyd
        rlIsRHEL '<8' && rlServiceStop ntpd
    fi
    sb_config_smb_conf_member
    sb_config_resolv_conf
    #if DNS is setup right then PDC can be discovered from it - commenting out
    sb_config_lmhosts
    sb_config_hosts
#comented out because samba can (and should) create its own krb5.conf (in /var/cache/samba/smb_krb)
#net leave/testjoin and nss-winbind accessing trusted domains doesn't support this - re-enabling
    sb_config_krb5_conf
    sb_pdc_time_synchro
    sb_config_nsswitch_conf
    sb_clean_cache
    if rlIsRHEL 5 6; then
        rlRun "which restorecon && runcon -t unconfined_t restorecon -v -R /etc/samba/" 0 "restoring selinux contexts on /etc/samba/**"
    fi
    rlRun "testparm -s" 0 "checking smb.conf sanity"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_join_method()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#usage: sb_join_method [ads|rpc]
sb_join_method(){
    if [ "$1" = "ads" ] ; then
        #ads join fails when NBNAME differs from hostname - work around it
        local NBNAME=`sb_get_nbname`
        local HOSTNAME=`hostname -s`
        if [ "$NBNAME" != "$HOSTNAME" ] ; then
            if ! grep -q "$NBNAME[ $]" /etc/hosts ; then
                rlFileBackup --namespace samba_bits /etc/hosts
                sed -i "/$HOSTNAME/ s/\\(.*\\)/\\1 $NBNAME/" /etc/hosts
            fi
        fi
    fi
    local JOIN_LOG=`mktemp`
    net $1 join -d 10 -U$PDC_LOGIN%$PDC_PASSWORD &> $JOIN_LOG &
    local NET_PID=$!
    sb_tries "ps h -p $NET_PID" 20 10 1 "waiting for net join to finish"
    #if still running
    if ps h -p $NET_PID ; then
        rlLogError "net join is still running, killing"
        kill -9 $NET_PID
        rlBundleLogs "net-join.log" $JOIN_LOG
    fi
    wait $NET_PID
    rlAssert0 "joined $REALM domain (method $1)" $?
    if [ $? -ne 0 ] ; then 
        rlBundleLogs "net-join.log" $JOIN_LOG
    fi
    cat $JOIN_LOG
    rm -f "$JOIN_LOG"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_leave()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_leave 

Function checks value of global parameter SECURITY. If value is ADS, function runs command X-times unless it returns result. 

=over

=back
	
=cut

sb_leave(){
    if [ "$SECURITY" == "ADS" ] ; then
	# -S is here for covering bz#1089940
        sb_tries "net ads leave -U${PDC_LOGIN}%${PDC_PASSWORD} -I ${PDC_IP} -S ${PDC_NBNAME}" 10 10 0 "leaving domain, hiding password"
    else
        rlLogWarning "can't leave domain - security is not ADS but $SECURITY"        
    fi
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_join()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_join 

This function call function sb_join_method with parameter ads or rpc according to value of global parameter SECURITY.

=over

=back
	
=cut

sb_join(){
    if [ "x$SECURITY" = "xADS" ] ; then
        sb_join_method "ads"
    else
        sb_join_method "rpc"
    fi
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_join_verify()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_join_verify 

Function checks if the membership in an Active Directory is valid.

=over

=back
	
=cut

sb_join_verify(){
    # skip this on rhel5
    if rlIsRHEL 5; then
        return 0
    fi
    if [ "x$SECURITY" = "xADS" ] ; then
        local METHOD="ads"
    else
        local METHOD="rpc"
    fi
    rlRun "net $METHOD testjoin -U$PDC_LOGIN%$PDC_PASSWORD" 0 "net $METHOD testjoin works"
    rlRun "wbinfo -t" 0 "trust secret is valid"

    wbinfo -u
    if rlIsRHEL '>=7.2'; then
	    rlRun "wbinfo -u --domain=$REALM | grep -i $PDC_LOGIN" 0 "user $PDC_LOGIN found in user listing"
    else
	    rlRun "wbinfo -u | grep -i $PDC_LOGIN" 0 "user $PDC_LOGIN found in user listing"
    fi
    local SEP=`sb_get_separator`
    getent passwd ${WORKGROUP}${SEP}${PDC_LOGIN}
    rlAssert0 "user ${WORKGROUP}${SEP}$PDC_LOGIN present in system" $?
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_config_add()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_config_add [SECTION OPTION|]

Function adds configuration into specified section of smb.conf.

=over

=item SECTION

Specified section in smb.conf. If parameter is not set, it is set to value global.

=item OPTION

Configuration option that will be added into specified section.

=back
	
=cut

#prida parametry do urcitych sekci

#add config option into specified section of smb.conf
sb_config_add(){
    local SECTION=${1:-global}
    local OPTION=$2
    sed -i "/\\[$SECTION\\]/ a${OPTION}" /etc/samba/smb.conf
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_config_modify()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_config_modify [SECTION KEY VALUE|]

This function finds key in specified section in smb.conf and replaces it with a new value. If the key is not set in the specified section, then nothing happens.

=over

=item SECTION

Specified section in smb.conf. If parameter is not set, it is set to value global.

=item KEY

Item that will be modified.

=item VALUE

New value replaces founded key.

=back
	
=cut

#params: section key value
#if key is not set in the specified section then nothing happens
sb_config_modify(){
    local SECTION=${1:-global}
    local KEY=$2
    local VALUE=$3
    sed -i '/\['"${SECTION}"'\].*/,/\[.*\]/ s!^ *'"${KEY}"' *=.*!'"${KEY}"' = '"${VALUE}"'!' /etc/samba/smb.conf
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_config_delete()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_config_delete [SECTION PATTERN|]

Function finds pattern in specified section in smb.conf and removes it.

=over

=item SECTION

Specified section in smb.conf. If parameter is not set, it is set to value global.

=item PATTERN

Item that will be deleted from specified section.

=back
	
=cut

#removes line matching pattern from specific section in smb.conf
#params: section pattern
sb_config_delete(){
    local SECTION=${1:-global}
    local PATTERN=$2
    sed -i '/\['"${SECTION}"'\].*/,/\[.*\]/ s!^.*'"${PATTERN}.*"'!!' /etc/samba/smb.conf
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_share_add()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_share_add [SHARE SHAREPATH|]

Function adds shared temporary directory into smb.conf. It creates new section in configuration file and adds settings of it. Also it takes care about SELinux context.

=over

=item SHARE

Name of shared component.

=item SHAREPATH

Path to shared temporary directory. If it is not specified, new temporary directory will be created.

=back
	
=cut

#adds file share to smb.conf
sb_share_add(){
    local SHARE=${1:-zelshare}
    local SHAREPATH=$2
    if [ -z "$SHAREPATH" ] ; then
        SHAREPATH=`mktemp -d`
    fi
    [ -d "$SHAREPATH" ] || mkdir "$SHAREPATH"
    chcon unconfined_u:object_r:samba_share_t:s0 $SHAREPATH
    chmod a+rwx "$SHAREPATH"

    echo "[$SHARE]" >> /etc/samba/smb.conf
    sb_config_add $SHARE "path = $SHAREPATH"
    sb_config_add $SHARE "read only  = no"
    rlLogInfo "addded share $SHARE with path $SHAREPATH"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_get_cache_dir()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_get_cache_dir

=over

=back

Return path of cache directory for currently installed Samba.

=cut

#echos correct cache dir for currently installed samba
sb_get_cache_dir(){
    local CACHE_DIR="/var/lib/samba"
    if rlIsRHEL 5 && rpm -q samba-common >/dev/null ; then
        CACHE_DIR="/var/cache/samba"
    fi
    echo $CACHE_DIR
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_clean_cache()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_clean_cache

Cleans Samba cache memory.

=over

=back

=cut

sb_clean_cache() {
    rlLogInfo "deleting samba caches"
    local CACHE_DIR=`sb_get_cache_dir`
    [ -d "$CACHE_DIR" ] &&  find "$CACHE_DIR" -name '*.tdb' -exec rm -f '{}' ';' || rlLogError "error cleaning caches"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_clean_etc_samba()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_clean_etc_samba

User credentials stored during testing and kerberos keys are removed.

=over

=back

=cut

sb_clean_etc_samba(){
    # remove user credentials stored during testing
    rm -f /etc/samba/secrets.tdb
    rm -f /etc/samba/passdb.tdb
    rm -f /etc/samba/smbpasswd
    # this file will be restored if exists before
    rm -f /etc/samba/lmhosts

    # also remove kerberos keys etc.
    kdestroy
    rm -f /etc/krb5.keytab
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_get_ip()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_get_ip

=over

=back

Return IPv4 address of machine where test runs.

=cut

sb_get_ip(){
    local DEVICE=`ip r |grep default | sed 's/default via.*dev \([^ ]*\).*/\1/'`
    local IP=`ip a show  "$DEVICE" |grep "inet " |sed 's/ *inet *\([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*\).*/\1/'`
    echo "$IP"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_win_script_config()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#replaces token ${$2} with value $3 in file(s) $1
sb_win_script_config(){
    local TOKEN=$2
    local VALUE=$3
    for FILE in $1 ; do
        [ -f "$FILE" ] || { rlAssert0 "win script \"$FILE\" does exist" 1 ; break ; }
        [ ! -f "$FILE".sws-orig ] && cp -f "$FILE" "$FILE".sws-orig
        sed -i "s/\${$TOKEN}/$VALUE/g" "$FILE"
        rlAssert0 "win script prepare successful (replacing $TOKEN with $VALUE in $FILE)" $?
    done
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_win_script_upload()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#takes IP of remote server as first arg and uploads file(s) (rest of arg list) to winland's script directory, under $TEST/subdir
sb_win_script_upload(){
    local NAME=`basename $TEST`
    local CMD="cd scripts/ ; del *"
    local IP="$1"
    for i in "${@:2}" ; do
        CMD=$CMD"; put $i"
    done
    rlLogDebug "sb_win_script_upload CMD: $CMD"
    smbclient "\\\\$IP\share" -U guest% -c "$CMD"
    rlAssert0 "uploading files to winland" $?
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_win_script_restore()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#restore win scripts to their backuped originals
sb_win_script_restore(){
    for i in *.sws-orig ; do
        [ "$i" == '*.sws-orig' ] && break 
        ORIG_NAME=`basename "$i" '.sws-orig'`
        rlRun "mv -f \"$i\" \"$ORIG_NAME\"" 0 "restoring win script '$i'"
    done
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_packages_remove()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_packages_remove PGKS_LIST [RPM_PARAMS|]

All Samba related packages are removed using rpm tool.

=over

=item PKGS_LIST

List with samba packages.

=item RPM_PARAMS

Optional parameters for rpm tool.

=back

=cut

sb_packages_remove(){
    local PKGS_LIST="$1"
    local RPM_PARAMS="${2:-""}"

    if ! [ -s $PKGS_LIST ] ; then
        rlLogWarning "List with packages was not given, skipping ..."
        return
    fi

    local LIST=""
    for pkg in `cat ${PKGS_LIST} | grep -v '^#' | sort | uniq` ; do
        rpm -q $pkg && LIST=$LIST" $pkg"
    done

    rlLogInfo "removing $LIST packages"
    rpm -e $RPM_PARAMS $LIST
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_packages_remove2()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_packages_remove2

Uninstall Samba with yum tool.

=over

=back

=cut

sb_packages_remove2(){
    # Never ever trust yum!
    # this function is very usefull on stable systems.
    yum -y remove `rpm -qa samba\*`
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_packages_list()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_packages_list PKGS_LIST

=over

=item PKGS_LIST

List with samba packages.

=back

Return versions of all known and installed Samba packages.

=cut

#queries all known samba packages for versions
sb_packages_list(){
    local PKGS_LIST="$1"
    if ! [ -s $PKGS_LIST ] ; then
        rlLogWarning "List with packages was not given, skipping ..."
        return
    fi

    for pkg in `cat ${PKGS_LIST} | grep -v '^#' | sort | uniq` ; do
        rpm -q $pkg
    done
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_packages_install()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_packages_install PKGS_LIST

Installs all packages from a specific list.

=over

=item PKGS_LIST

List with samba packages.

=back

=cut

sb_packages_install(){
    local PKGS_LIST="$1"
    if ! [ -s $PKGS_LIST ] ; then
        rlLogWarning "List with packages was not given, skipping ..."
        return
    fi

    local LIST=""
    for pkg in `cat {PKGS_LIST} | grep -v '^#' | sort | uniq` ; do
        LIST=$LIST" $pkg"
    done
    rlRun "yum install -y $LIST" 0 "installing $LIST packages"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_wait_all()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_wait_all

Executes sb_wait_smbd and sb_wait_winbind. See below for more information.

=over

=back

=cut

sb_wait_all() {
    sb_wait_smbd
    sb_wait_winbind
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_wait_winbind()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_wait_winbind

Actively waits untill winbind daemon starts working.

=over

=back

=cut

sb_wait_winbind() {
    ret=0

    rlLog "Waiting for winbind"
    for ((count = 0; count < 60; ++count)); do
        smbcontrol winbind ping &> /dev/null
        ret=$?
        if [ $ret -eq 0 ]; then
            break
        fi
        sleep 1
    done
    if [ $ret -eq 0 ]; then
        rlPass "PASS: Winbind is running"
        # This sleep helps from test failling, when
        # sb_join_verify is called right after this function
        sleep 5
        return
    fi
    rlFail "ERROR: waiting for winbind to start"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_wait_smbd()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_wait_smbd

Actively waits untill smb daemon starts working.

=over

=back

=cut

sb_wait_smbd() {
    smbcontrol smbd ping &> /dev/null
    ret=$?
    if [ $ret -eq 127 ]; then
        rlFail "No 'smbclient' command available!"
        return
    # We dont have to run same command in the loop if first smbclient succeeded
    elif [ $ret -eq 0 ]; then
	rlPass "smbd: SMBD started"
        return
    fi

    for ((count = 0; count < 20; ++count)); do
        smbcontrol smbd ping &> /dev/null
        ret=$?
        if [ $ret -eq 0 ]; then
            break
        fi
        sleep 1
    done
    if [ $ret -eq 0 ]; then
        rlPass "smbd: SMBD started"
        return
    fi
    rlFail "ERROR: smbd not running"
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_tries()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_tries

Function tries to run command X-times unless it returns value.

=over

=item COMMAND

Command to run.

=item COUNT

Number how many times will try to run the command.

=item PAUSE

Time how long the function sleep before another attempt.

=item RESULT

Value of the expected result of the command.

=item DESCRIPTON

=back

Returns result of specified command.

=cut

#runs command X-times unless it returns $RESULT
sb_tries(){
    local COMMAND=$1
    local COUNT=${2:-10}
    local PAUSE=${3:-10}
    local RESULT=${4:-0}
    local DESCRIPTION=${5:-$COMMAND}
    local ECODE=0
    while [ $COUNT -gt 0 ] ; do
        echo "tries: COUNT=$COUNT PAUSE=$PAUSE RESULT=$RESULT COMMAND=$DESCRIPTION"
        eval $COMMAND
        ECODE=$?
        [ $ECODE -eq $RESULT ] && return 0
        echo "ecode: $ECODE"
        sleep $PAUSE
        COUNT=$(( $COUNT -1 ))
    done
    return $ECODE
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: sb_deploy_aux()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sb_deploy_aux

This function creates support files for samba tests. 

=over

=back

SSH key is ready to be used via $SB_SSH_KEY_PATH variable.

=cut

sb_deploy_aux(){
    # sshpass.exp -> call ssh via expect script and run command
    cat > ./sshpass.exp <<-EOF
#!/usr/bin/expect -f
#  ./ssh.exp password 192.168.1.11 username id
set pass [lindex \$argv 0]
set server [lindex \$argv 1]
set name [lindex \$argv 2]
set command [lindex \$argv 3]

set timeout 60

spawn ssh -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null -l \$name \$server "\$command"
match_max 100000
expect {
        "*?assword:*" {}
        timeout { exit 56 }
}
send -- "\$pass\r"
send -- "\r"
expect {
        "Permission denied" { exit 55 }
        timeout { exit 56 }
}
EOF

    chmod +x "./sshpass.exp"

    #Find dir location of samba-bits lib
    DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
    source $DIR/modules/private/key.sh $DIR
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: __sb_fini()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 __sb_fini

Samba-bits library destructor. Destructor cleans environment. For more details, see the function sb_restore_and_set_clean_env.

=over

=back

=cut

# this is samba-bits library destructor
__sb_fini(){
    echo -e "\n\n\n"
    rlLogInfo "Samba-bits destructor hit."
    sb_restore_and_set_clean_env
    # Always end with positive result.
    rlLogInfo "Samba-bits: destructor FINISHED."
    exit 0
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Function: __sb_init()
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 __sb_init

Samba-bits library constructor. Constructor cleans environment. For more details, see the function sb_save_and_set_clean_env.

=over

=back

=cut

# This is samba-bits library constructor.
__sb_init(){
    rlLogInfo "Samba-bits constructor hit."

    # This is very old constructor code...
############################################################
    # Pre constructor
    rlLog "samba-bits functions v$SB_VERSION included"
    #sb_packages_list -> TODO how to solve problem with argument, how and where keep list with packages
    sb_win_script_restore
    (rpm -q ntpdate || rpm -q ntp) && {
        rlRun "rpm -qf $(which ntpdate)" 0 "check presence of the ntpdate binary"
    } || rlRun "rpm -q chrony" 0 "check presence of the chronyd binary"

    #workaround for: ppc64 is preferred to ppc when satisfying deps
    rlIsRHEL 5 && rpm -q samba-common && [ `uname -i` == 'ppc64' ] && {
        rlRun "rpm -q samba-common.ppc samba-common.ppc64 | grep ^samba-common | wc -l | grep ^2$ || yum install  -y samba-common.{ppc,ppc64}" 0 "BZ#705851 workaround - installing samba-common.{ppc,ppc64}"
    } || /bin/true
############################################################
    # Disable new type of constructor - this can interrferr with samba-branch-switch
    if [[ -z $SB_DISABLE_NEW_CONST ]]; then
        sb_deploy_aux
        sb_save_and_set_clean_env
    fi

    rlLogInfo "Samba-bits: constructor FINISHED."
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Main 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# This tweak should speedup start of samba
export TDB_NO_FSYNC=1
# Destructor and constructor are called only if they run under make run.
if [[ ! -z $MAKELEVEL ]]; then
    __sb_init
    # register destructor - disable it when we need that
    if [[ -z $SB_DISABLE_NEW_CONST ]]; then
        trap "__sb_fini" EXIT
    fi
fi


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Execution
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Verification
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Authors
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head1 AUTHORS

=over

=item *

Ales Zelinka <azelinka@redhat.com>

=item *

Robin Hack <rhack@redhat.com>

=item *

Tereza Cerna <tcerna@redhat.com>

=item *

Andrej Dzilsky <adzilsky@redhat.com>

=back

=cut
